package ip.hak;

import futils.Futil;
import j2d.ImageUtils;
import j2d.ShortImageBean;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class HaarLiftingFrame extends Frame
        implements ActionListener {
    MenuBar mb = new MenuBar();
    Menu fMenu = new Menu("File");
    MenuItem open_MI, quit_MI;
    Image img = null, xfimg[];
    String fname = null, dir = null;
    RecieveFrame rf;
    Button svButton, sdButton;
    ShortImageBean sib;


    public static void main(String args[]) {
        HaarLiftingFrame hlf = new HaarLiftingFrame("Haar Lifting Frame");
        hlf.setVisible(true);
    }

    public HaarLiftingFrame(String title) {
        super(title);
        init();
    }

    public void init() {
        setLayout(null);
        int c = 203;
        Color mc = new Color(c, c, c);
        setBackground(mc);
        setSize(380, 320);
        open_MI = new MenuItem("Open Image File..");
        fMenu.add(open_MI);
        open_MI.addActionListener(this);
        quit_MI = new MenuItem("Quit");
        fMenu.add(quit_MI);
        quit_MI.addActionListener(this);
        mb.add(fMenu);
        setMenuBar(mb);

        svButton = new Button("Save");
        svButton.setSize(50, 20);
        svButton.setLocation(300, 100);
        add(svButton);
        svButton.setEnabled(false);
        svButton.addActionListener(this);

        sdButton = new Button("Send");
        sdButton.setSize(50, 20);
        sdButton.setLocation(300, 140);
        add(sdButton);
        sdButton.setEnabled(false);
        sdButton.addActionListener(this);
    }

    public void openFile() {

        fname = Futil.getReadFileName();
        if (fname == null) {
            dir = null;
            return;
        }
        openImage(fname);
    }

    public void openImage(String fn) {
        img = ImageUtils.getImage(fname);
        repaint();

        sib = new ShortImageBean(img);
        rf = new RecieveFrame("Recieve Frame", sib.getWidth(), sib.getHeight());
        n = 2;
        math.wavelets.Lifting.forwardHaar(sib.getR());
        math.wavelets.Lifting.forwardHaar(sib.getG());
        math.wavelets.Lifting.forwardHaar(sib.getB());

        svButton.setEnabled(true);
        sdButton.setEnabled(true);
    }


    public void paint(Graphics g) {
        Rectangle rec = getBounds();
        if (img == null) {
            g.drawRect(20, rec.height - 276, 256, 256);
            return;
        }

        g.drawImage(img, 20, rec.height - 276, 256, 256, this);

    }

    int n = 1;

    public void send() {
        rf.initArrays();
        rf.sendData(sib.getR(), sib.getG(), sib.getB(), n);
        n *= 2;
        if (n > sib.getHeight()) {
            sdButton.setEnabled(false);
            sdButton.transferFocus();
        }
    }

    public void saveFile() {
        FileDialog fd = new FileDialog(this,
                "Save as lzc file",
                FileDialog.SAVE);
        fd.setFile("images.lzc");
        fd.setVisible(true);
        String fname = fd.getFile();
        if (fname != null) {
            String dir = fd.getDirectory();
            saveFile(dir + fname);
        }
    }

    public void saveFile(String fn) {
        try {
            FileOutputStream fos = new FileOutputStream(fn);
            ZipOutputStream zos = new ZipOutputStream(fos);
            ObjectOutputStream oos;
            int sn = 0;
            int en = 1;
            int num = 0;
            while (en <= sib.getHeight()) {
                ZipEntry ze = new ZipEntry(makeName(num++));
                zos.putNextEntry(ze);
                oos = new ObjectOutputStream(zos);
                saveObject(oos, sn, en);
                sn = en;
                en *= 2;
            }

            zos.finish();
            zos.close();
            System.out.println("Save done...");
        } catch (IOException e) {
            System.out.println(e);
        }
    }

    public String makeName(int i) {
        String name1 = "Datafile";
        String name2;
        if (i < 10)
            name2 = "00" + i + ".dat";
        else if (i < 100)
            name2 = "0" + i + ".dat";
        else
            name2 = "" + i + ".dat";

        return name1 + name2;
    }

    public void saveObject(ObjectOutputStream oos, int sind, int eind) {
        int size = eind - sind;
        int w = sib.getWidth();
        short rt[][] = new short[w][size];
        short gt[][] = new short[w][size];
        short bt[][] = new short[w][size];

        for (int x = 0; x < w; x++)
            for (int y = sind, si = 0; y < eind; y++, si++) {
                rt[x][si] = sib.getR()[x][y];
                gt[x][si] = sib.getG()[x][y];
                bt[x][si] = sib.getB()[x][y];
            }
        try {
            oos.writeObject(rt);
            oos.writeObject(gt);
            oos.writeObject(bt);
        } catch (IOException e) {
            System.out.println(e);
        }

    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == open_MI) {
            openFile();
            return;
        }

        if (e.getSource() == quit_MI) {

            dispose();
            return;
        }
        if (e.getSource() == svButton) {
            saveFile();
            return;
        }
        if (e.getSource() == sdButton) send();
    }
}